(function (global, factory) {
    if (typeof define === "function" && define.amd) {
        define(['exports', 'react', 'lodash.assign', 'lodash.noop', 'lodash.omit'], factory);
    } else if (typeof exports !== "undefined") {
        factory(exports, require('react'), require('lodash.assign'), require('lodash.noop'), require('lodash.omit'));
    } else {
        var mod = {
            exports: {}
        };
        factory(mod.exports, global.react, global.lodash, global.lodash, global.lodash);
        global.ReactTouchPosition = mod.exports;
    }
})(this, function (exports, _react, _lodash, _lodash3, _lodash5) {
    'use strict';

    Object.defineProperty(exports, "__esModule", {
        value: true
    });

    var _react2 = _interopRequireDefault(_react);

    var _lodash2 = _interopRequireDefault(_lodash);

    var _lodash4 = _interopRequireDefault(_lodash3);

    var _lodash6 = _interopRequireDefault(_lodash5);

    function _interopRequireDefault(obj) {
        return obj && obj.__esModule ? obj : {
            default: obj
        };
    }

    exports.default = _react2.default.createClass({

        displayName: 'TouchPosition',

        getInitialState: function getInitialState() {
            return {
                isHovering: false,
                isTouchOutside: false,
                cursorPosition: {
                    x: 0,
                    y: 0
                }
            };
        },


        propTypes: {
            className: _react.PropTypes.string,
            isActivatedOnTouch: _react.PropTypes.bool,
            onActivationChanged: _react.PropTypes.func,
            onPositionChanged: _react.PropTypes.func,
            pressDuration: _react.PropTypes.number,
            pressMoveThreshold: _react.PropTypes.number,
            shouldDecorateChildren: _react.PropTypes.bool,
            style: _react.PropTypes.object
        },

        getDefaultProps: function getDefaultProps() {
            return {
                isActivatedOnTouch: false,
                onActivationChanged: _lodash4.default,
                onPositionChanged: _lodash4.default,
                pressDuration: 500,
                pressMoveThreshold: 5,
                shouldDecorateChildren: true
            };
        },
        onTouchStart: function onTouchStart(e) {
            this.elementOffsetRect = this.getViewportRelativeElementRect(e.currentTarget);
            this.setPosition(e);

            if (this.props.isActivatedOnTouch) {
                e.preventDefault();

                this.setState({
                    isHovering: true
                });

                return;
            }

            this.initPressEventCriteria(e.touches[0]);

            this.setPressEventTimer();
        },
        onTouchMove: function onTouchMove(e) {
            this.setPressEventCriteria(e.touches[0]);

            if (!this.state.isHovering) {
                return;
            }

            this.setPosition(e);

            e.preventDefault();
        },
        onTouchEnd: function onTouchEnd() {
            this.clearTimers();

            this.setState({
                isHovering: false,
                isTouchOutside: false
            });

            this.props.onActivationChanged({ isHovering: false });
        },
        setPosition: function setPosition(e) {
            var viewportRelativeTouchPosition = this.getViewportRelativeTouchPosition(e);
            var elementOffsetRect = this.elementOffsetRect;
            var cursorPosition = this.getElementRelativeTouchPosition(viewportRelativeTouchPosition, elementOffsetRect);
            var isPositionOutside = this.getIsPositionOutside(viewportRelativeTouchPosition, elementOffsetRect);

            this.setState({
                cursorPosition: cursorPosition,
                isPositionOutside: isPositionOutside
            });

            this.props.onPositionChanged(Object.assign({ isPositionOutside: isPositionOutside }, cursorPosition));
        },
        setPressEventTimer: function setPressEventTimer() {
            var _this = this;

            this.pressDurationTimerId = setTimeout(function () {
                if (Math.abs(_this.currentElTop - _this.initialElTop) < _this.props.pressMoveThreshold) {
                    _this.setState({ isHovering: true });
                    _this.props.onActivationChanged({ isHovering: true });
                }
            }, this.props.pressDuration);
        },
        setPressEventCriteria: function setPressEventCriteria(touch) {
            if (!this.props.isActivatedOnTouch) {
                if (!this.state.isHovering) {
                    this.currentElTop = touch.clientY;
                } else {
                    this.initialElTop = touch.clientY;
                }
            }
        },
        initPressEventCriteria: function initPressEventCriteria(touch) {
            var top = touch.clientY;
            this.initialElTop = top;
            this.currentElTop = top;
        },
        getViewportRelativeElementRect: function getViewportRelativeElementRect(el) {
            return el.getBoundingClientRect();
        },
        getIsPositionOutside: function getIsPositionOutside(viewportRelativeTouchPosition, elementOffsetRect) {
            var viewportRelativeTouchX = viewportRelativeTouchPosition.x;
            var viewportRelativeTouchY = viewportRelativeTouchPosition.y;
            var offsetTop = elementOffsetRect.top;
            var offsetRight = elementOffsetRect.right;
            var offsetBottom = elementOffsetRect.bottom;
            var offsetLeft = elementOffsetRect.left;

            return viewportRelativeTouchX < offsetLeft || viewportRelativeTouchX > offsetRight || viewportRelativeTouchY < offsetTop || viewportRelativeTouchY > offsetBottom;
        },
        getViewportRelativeTouchPosition: function getViewportRelativeTouchPosition(event) {
            var touch = event.touches[0];
            return {
                x: touch.clientX,
                y: touch.clientY
            };
        },
        getElementRelativeTouchPosition: function getElementRelativeTouchPosition(viewportRelativetouchPosition, elementOffsetRect) {
            var touchX = viewportRelativetouchPosition.x;
            var touchY = viewportRelativetouchPosition.y;
            var offsetX = elementOffsetRect.left;
            var offsetY = elementOffsetRect.top;


            return {
                x: touchX - offsetX,
                y: touchY - offsetY
            };
        },
        isReactComponent: function isReactComponent(reactElement) {
            return typeof reactElement.type === 'function';
        },
        shouldDecorateChild: function shouldDecorateChild(child) {
            return this.isReactComponent(child) && this.props.shouldDecorateChildren;
        },
        decorateChild: function decorateChild(child, props) {
            return (0, _react.cloneElement)(child, props);
        },
        renderChildrenWithProps: function renderChildrenWithProps(children, props) {
            var _this2 = this;

            return _react.Children.map(children, function (child) {
                return _this2.shouldDecorateChild(child) ? _this2.decorateChild(child, props) : child;
            });
        },
        clearTimers: function clearTimers() {
            clearTimeout(this.pressDurationTimerId);
        },
        componentWillUnmount: function componentWillUnmount() {
            this.clearTimers();
        },
        render: function render() {
            var _props = this.props;
            var children = _props.children;
            var className = _props.className;
            var style = _props.style;
            var _state = this.state;
            var isHovering = _state.isHovering;
            var isTouchOutside = _state.isTouchOutside;
            var cursorPosition = _state.cursorPosition;

            var childProps = (0, _lodash2.default)({}, {
                isHovering: isHovering,
                isTouchOutside: isTouchOutside,
                cursorPosition: cursorPosition
            }, (0, _lodash6.default)(this.props, ['children', 'className', 'isActivatedOnTouch', 'onActivationChanged', 'onPositionChanged', 'onTouchOutside', 'pressDuration', 'pressMoveThreshold', 'shouldDecorateChildren', 'style']));

            return _react2.default.createElement(
                'div',
                {
                    onTouchStart: this.onTouchStart,
                    onTouchMove: this.onTouchMove,
                    onTouchEnd: this.onTouchEnd,
                    className: className,
                    style: Object.assign({}, style, {
                        WebkitUserSelect: 'none'
                    })
                },
                this.renderChildrenWithProps(children, childProps)
            );
        }
    });
});